@@ -46,7 +46,7 @@ struct Listener {
 
 };
 
-void listeners_initialise();
+void listeners_initialise(void);
 void handle_listeners(fd_set * readfds);
 void set_listener_fds(fd_set * readfds);
 
