@@ -82,7 +82,7 @@ struct SIGN_key {
 
 typedef struct SIGN_key sign_key;
 
-sign_key * new_sign_key();
+sign_key * new_sign_key(void);
 const char* signkey_name_from_type(enum signkey_type type, unsigned int *namelen);
 enum signkey_type signkey_type_from_name(const char* name, unsigned int namelen);
 int buf_get_pub_key(buffer *buf, sign_key *key, enum signkey_type *type);
